#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <stdio.h>


#include "defines.h"
#include "graphics.h"

uint8_t fgcolor=7, bkcolor=0;
uint8_t* current_font = 0;



void PutPixelColor (uint16_t x, uint16_t y, uint8_t color)
{
	x = 320-1-x;
	y = 240-1-y;

	int i;

	for (i=0; i<3; i++)
	{
		uint16_t nbit = (x+x+x+i);
		uint16_t nbyte = nbit>>3;
		nbit = 7-(nbit&7);

		uint16_t addr = (y<<7) + nbyte;

		if (color & 4)
			WriteExtRam( addr, ReadExtRam(addr) | (1<<nbit) );
		else
			WriteExtRam( addr, ReadExtRam(addr) & (~(1<<nbit)) );

		color <<= 1;
	}
}



void ClearScreen(void)
{
	uint16_t row;
	for (row = 0; row<240; row++)
	{
		uint16_t addr = (row<<7);
		uint16_t nbyte;
		for (nbyte=0; nbyte<120; nbyte+=3)
		{
			WriteExtRam(addr++, (bkcolor<<5) | (bkcolor<<2) | (bkcolor>>1));
			WriteExtRam(addr++, (bkcolor<<7) | (bkcolor<<4) | (bkcolor<<1) | (bkcolor>>2));
			WriteExtRam(addr++, (bkcolor<<6) | (bkcolor<<3) | (bkcolor>>0));

		}
	}
}


void DrawXLine(uint16_t x1, uint16_t x2, uint16_t y)
{	
	uint16_t x;
	if (x1>x2)
	{
		uint16_t t;
		t = x1;
		x1 = x2;
		x2 = t;
	}
	for (x=x1; x<=x2; x++)
		SetPixel(x,y);
}

void DrawXLineDotted(uint16_t x1, uint16_t x2, uint16_t y)
{
	uint16_t x;
	if (x1>x2)
	{
		uint16_t t;
		t = x1;
		x1 = x2;
		x2 = t;
	}
	for (x=x1; x<=x2; x+=2)
		SetPixel(x,y);
}

void DrawYLine(uint16_t x, uint16_t y1, uint16_t y2)
{
	uint16_t y;
	if (y1>y2)
	{
		uint8_t t;
		t = y1;
		y1 = y2;
		y2 = t;
	}
	for (y=y1; y<=y2; y++)
		SetPixel(x,y);
}

void DrawYLineDotted(uint16_t x, uint16_t y1, uint16_t y2)
{
	uint16_t y;
	if (y1>y2)
	{
		uint8_t t;
		t = y1;
		y1 = y2;
		y2 = t;
	}
	for (y=y1; y<=y2; y+=2)
		SetPixel(x,y);
}

void Border(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	DrawXLine(x1,x2,y1);
	DrawXLine(x1,x2,y2);
	DrawYLine(x1,y1,y2);
	DrawYLine(x2,y1,y2);
}

void FillRect(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	int16_t y;
	for (y=y1; y<=y2; y++)
		DrawXLine(x1,x2,y);
}

uint8_t* SetCurrentFont(uint8_t* new_font)
{
	uint8_t* tmp = current_font;
	current_font = new_font;

	return tmp;
}

uint8_t* GetCurrentFont(void)
{
	return current_font;
}


uint16_t PutChar(uint16_t x, uint16_t y, char c)
{

	uint16_t i, j, width, height;
	uint16_t t;
	const prog_uchar *p;
	uint8_t* font_in_use = current_font;


#ifdef _DEBUG
	if ( ((uint8_t)c < pgm_read_byte(current_font+5)) ||
		((uint8_t)c > pgm_read_byte(current_font+6)) )
		font_in_use = f8x8;

#endif

	c -= pgm_read_byte(font_in_use+5); 
	width = pgm_read_byte(font_in_use+7+(uint8_t)c);
	height = pgm_read_byte(font_in_use+3);

	p = font_in_use+7+pgm_read_byte(font_in_use+6) - pgm_read_byte(font_in_use+5)+1;

	if (height==8)
	{
		for (i=0; i<(uint8_t)c; i++)
			p += pgm_read_byte(font_in_use+7+i);

//	#ifdef WIN32
//		TRACE("(w=%i)\n", width);
//	#endif
		for (i=0; i<width; i++)
		{
			c = pgm_read_byte(p++);

			for (j=0; j<8; j++)
			{
				PutPixel(x,y+j,(c>>j)&1);
//	#ifdef WIN32
//				if ((c>>j)&1)
//					TRACE("*");
//				else
//					TRACE(".");
//	#endif
			}
			x++;
//	#ifdef WIN32
//		TRACE("\n");
//	#endif
		}

		for (j=0; j<8; j++)
		{
			PutPixel(x, y+j, 0);
		}

		return width+1;
	}

	t = 0;
	for (i=0; i<(uint8_t)c; i++)
		t += pgm_read_byte(font_in_use+7+i);

	int t1 = t*height/8;
	int t2 = t*height%8;

	p+=t1;

//	#ifdef WIN32
//		TRACE("(w=%i)\n", width);
//	#endif
	c = pgm_read_byte(p++);

	for (i=0; i<width; i++)
		{

			for (j=0; j<height; j++)
			{
				PutPixel(x,y+j,(c>>t2)&1);
//	#ifdef WIN32
//				if ((c>>t2)&1)
//					TRACE("*");
//				else
//					TRACE(".");
//	#endif
				t2++;
				t2 %= 8;
				if (t2==0)
				{
					t1++;
					c = pgm_read_byte(p++);
				}
			}
			x++;
//	#ifdef WIN32
//		TRACE("\n");
//	#endif
		}

		for (j=0; j<height; j++)
		{
			PutPixel(x, y+j, 0);
		}
		return width+1;
}

uint16_t PutText(uint16_t x, uint16_t y, const char* s)
{
	uint16_t width = 0;

	while(*s)
	{
		width += PutChar(x+width, y, *s++);
	}

	return width;
}

uint16_t PutTextP(uint16_t x, uint16_t y, const char* s)
{
	uint16_t width = 0;

	while(pgm_read_byte(s))
	{
		width += PutChar(x+width, y, pgm_read_byte(s++));
	}

	return width;
}

/*
uint16_t PutTextCenterP(uint16_t x, uint16_t y, const char* s)
{
		uint8_t width = GetWidthTextP(s);
		return PutTextP(x-(width>>1),y,s);
}
*/

void DrawLine(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	uint16_t nTmp;
	uint16_t nAlt=0;
	int x,y;        // where is the current pixel.
	int dx;   	// dx is the delta for x
	int dy;   	// dy is the delta for y
	int StepVal=0;  // variable for figuring out when to increment the other axis.
	if (x1>x2 && y1>y2)
	{
		nTmp=x2;
		x2=x1;
		x1=nTmp;
		
		nTmp=y2;
		y2=y1;
		y1=nTmp;

		dx=x2-x1;   // dx is the delta for x
		dy=y2-y1;   // dy is the delta for y
	}else
	{
		dx=x2-x1;   // dx is the delta for x
		dy=y2-y1;   // dy is the delta for y
		
		if (dy<0)
		{	
			dy=-dy;
			
			nTmp=y2;
			y2=y1;
			y1=nTmp;
			
			nAlt=1;
		}else
			if (dx<0)
			{	
				dx=-dx;
				
				nTmp=x2;
				x2=x1;
				x1=nTmp;
				
				nAlt=1;
			}
	}
	
	if (nAlt)
	{

		if(dx>=dy)       // The slope is less than 45 degres
		{
			y=y2;
			for(x=x1; x<=x2; x++)
			{
				// Call your function to draw a pixel here.
				SetPixel(x,y);
				StepVal+=dy;
				if(StepVal>=dx) // Increment y if enough x steps
								// have been taken.
				{
					y--;
					StepVal-=dx;    // Reset StepVal, but  
					// not to 0.  This gives even slopes.
				}
			}
		}
		else    // The slope is greater than 45 degrees, just like 
				// above, but with y instead of x.
		{
			x=x2;
			for(y=y1; y<=y2; y++)
			{
				// Call your function to draw a pixel here.
				SetPixel(x,y);
				StepVal+=dx;
				if(StepVal>=dy)
				{
					x--;
					StepVal-=dy;
				}
			}
		}
		return;
	}
	
	if(dx>=dy)       // The slope is less than 45 degres
	{
		y=y1;
		for(x=x1; x<=x2; x++)
		{
			// Call your function to draw a pixel here.
			SetPixel(x,y); 
			StepVal+=dy;
			if(StepVal>=dx) // Increment y if enough x steps
							// have been taken.
			{
				y++;
				StepVal-=dx;    // Reset StepVal, but  
				// not to 0.  This gives even slopes.
			}
		}
	}
	else    // The slope is greater than 45 degrees, just like 
			// above, but with y instead of x.
	{
		x=x1;
		for(y=y1; y<=y2; y++)
		{
			// Call your function to draw a pixel here.
			SetPixel(x,y);
			StepVal+=dx;
			if(StepVal>=dy)
			{
				x++;
				StepVal-=dy;
			}
		}
	}
	return;
}


