#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <compat/ina90.h>


#include "defines.h"
#include "keypad.h"


void KeypadOn(void)
{

	setbit(C1DDR, C1BIT);
	setbit(C2DDR, C2BIT);
	setbit(C3DDR, C3BIT);

	clrbit(C1PORT, C1BIT);
	clrbit(C2PORT, C2BIT);
	clrbit(C3PORT, C3BIT);

	clrbit(R1DDR, R1BIT);
	clrbit(R2DDR, R2BIT);
	clrbit(R3DDR, R3BIT);
	clrbit(R4DDR, R4BIT);

	setbit(R1PORT, R1BIT);
	setbit(R2PORT, R2BIT);
	setbit(R3PORT, R3BIT);
	setbit(R4PORT, R4BIT);
}

void KeypadOff(void)
{
	clrbit(C1DDR, C1BIT);
	clrbit(C2DDR, C2BIT);
	clrbit(C3DDR, C3BIT);

	clrbit(C1PORT, C1BIT);
	clrbit(C2PORT, C2BIT);
	clrbit(C3PORT, C3BIT);

	clrbit(R1DDR, R1BIT);
	clrbit(R2DDR, R2BIT);
	clrbit(R3DDR, R3BIT);
	clrbit(R4DDR, R4BIT);

	clrbit(R1PORT, R1BIT);
	clrbit(R2PORT, R2BIT);
	clrbit(R3PORT, R3BIT);
	clrbit(R4PORT, R4BIT);
}

#define KEYSCAN_DELAY 200

uint8_t GetKeyCode(void)
{

	// Col 1

	clrbit(C1PORT, C1BIT);
	setbit(C2PORT, C2BIT);
	setbit(C3PORT, C3BIT);

	_delay_loop_2(KEYSCAN_DELAY);

	if (getbit(R4PIN,R4BIT)==0)
		return KEY_STAR;
	if (getbit(R3PIN,R3BIT)==0)
		return KEY_7;
	if (getbit(R2PIN,R2BIT)==0)
		return KEY_4;
	if (getbit(R1PIN,R1BIT)==0)
		return KEY_1;

	// Col 2

	setbit(C1PORT, C1BIT);
	clrbit(C2PORT, C2BIT);
	setbit(C3PORT, C3BIT);

	_delay_loop_2(KEYSCAN_DELAY);

	if (getbit(R4PIN,R4BIT)==0)
		return KEY_0;
	if (getbit(R3PIN,R3BIT)==0)
		return KEY_8;
	if (getbit(R2PIN,R2BIT)==0)
		return KEY_5;
	if (getbit(R1PIN,R1BIT)==0)
		return KEY_2;

	// Col 3

	setbit(C1PORT, C1BIT);
	setbit(C2PORT, C2BIT);
	clrbit(C3PORT, C3BIT);

	_delay_loop_2(KEYSCAN_DELAY);

	if (getbit(R4PIN,R4BIT)==0)
		return KEY_SHARP;
	if (getbit(R3PIN,R3BIT)==0)
		return KEY_9;
	if (getbit(R2PIN,R2BIT)==0)
		return KEY_6;
	if (getbit(R1PIN,R1BIT)==0)
		return KEY_3;

	
	return KEY_NONE;
}

uint8_t WaitForKeyGetCode(void)
{
	uint8_t c;
	while (KEY_NONE == (c = GetKeyCode()))
	{
		delay_ms(25);

//		DecBacklitCntr();
	}

//	BacklitOn();
	return c;
}

void WaitForKeyReleased(void)
{
/*
	while (KEY_NONE == GetKeyCode())
	{
		CheckPrintScreen();
	}
*/
	clrbit(C1PORT, C1BIT);
	clrbit(C2PORT, C2BIT);
	clrbit(C3PORT, C3BIT);

	_delay_loop_2(KEYSCAN_DELAY);

	while( (getbit(R1PIN,R1BIT)==0) ||
			(getbit(R2PIN,R2BIT)==0) ||
			(getbit(R3PIN,R3BIT)==0) ||
			(getbit(R4PIN,R4BIT)==0) )
	{

	}

}
