#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <stdio.h>


#include "defines.h"
#include "usb.h"

void UsbInit(void)
{
	USBDDR = 0;
	USBPORT = 0;

	setbit(URDDDR, URDBIT);
	setbit(UWRDDR, UWRBIT);
	clrbit(UTXEDDR, UTXEBIT);
	clrbit(URXFDDR, URXFBIT);

	setbit(URDPORT, URDBIT);
	clrbit(UWRPORT, UWRBIT);
	clrbit(UTXEPORT, UTXEBIT);
	clrbit(URXFPORT, URXFBIT);

}

void UsbSendByte(uint8_t byte)
{
	while(getbit(UTXEPIN, UTXEBIT));

	USBDDR = 0xFF;
	USBPORT = byte;

	setbit(UWRPORT, UWRBIT);
	clrbit(UWRPORT, UWRBIT);

	USBDDR = 0;
	USBPORT = 0;
}

uint8_t UsbReceiveByte(void)
{
	setbit(URDPORT, URDBIT);
	uint8_t tmp = USBPIN;
	clrbit(URDPORT, URDBIT);

	return tmp;
}

uint8_t UsbIsByteReady(void)
{
	return !getbit(URXFPIN, URXFBIT);
}

void UsbSendText(const char* s)
{
	while(*s)
	{
		UsbSendByte(*s++);
	}
}

void UsbSendTextP(const char* s)
{
	while(pgm_read_byte(s))
	{
		UsbSendByte(pgm_read_byte(s++));
	}
}

void PrintScreen(void)
{

	uint16_t i;

	char s[3];

	UsbSendByte(13);
	UsbSendByte(10);

	UsbSendTextP(PSTR("screenshot: "));

	for (i=0;; i++)
	{
		UsbSendByte(ReadExtRam(i));
/*
		sprintf_P(s, PSTR("%02x"), (uint16_t)(ReadExtRam(i)));
		UsbSendByte(s[0]);
		UsbSendByte(s[1]);
*/
//		SerialTransmit(screen[i]);

		if (i==65535)
			break;
	}

	UsbSendByte(13);
	UsbSendByte(10);

}
